@@ -3,12 +3,12 @@
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Optional, Sequence  # type: ignore
+from typing import TYPE_CHECKING, Optional, Sequence
 from itertools import chain
 from collections import Counter
 
 import numpy as np
-from simpy.resources.store import FilterStore, FilterStoreGet  # type: ignore
+from simpy.resources.store import FilterStore, FilterStoreGet
 
 from wombat.core import (
     Failure,
@@ -22,6 +22,7 @@ from wombat.core import (
 if TYPE_CHECKING:
     from wombat.core import Port, ServiceEquipment
     from wombat.windfarm import Windfarm
+    from wombat.windfarm.system import Cable, System
 
 
 class RepairManager(FilterStore):
@@ -143,13 +144,19 @@ class RepairManager(FilterStore):
                 if operating_capacity > equipment.strategy_threshold:
                     continue
                 if capability in ("TOW", "AHV"):
+                    # Don't dispatch a second piece of equipment for tow-to-port
+                    if (
+                        request.system_id in self.invalid_systems
+                        and capability == "TOW"
+                    ):
+                        continue
                     try:
                         self.env.process(equipment.equipment.run_unscheduled(request))
                     except ValueError:
                         # ValueError is raised when a duplicate request is called for any of
                         # the port-based servicing equipment
                         pass
-                if equipment.equipment.onsite or equipment.equipment.enroute:
+                if equipment.equipment.dispatched:
                     continue
                 self.env.process(equipment.equipment.run_unscheduled(request))
 
@@ -168,6 +175,12 @@ class RepairManager(FilterStore):
                 # Run only the first piece of equipment in the mapping list, but ensure
                 # that it moves to the back of the line after being used
                 if capability in ("TOW", "AHV"):
+                    # Don't dispatch a second piece of equipment for tow-to-port
+                    if (
+                        request.system_id in self.invalid_systems
+                        and capability == "TOW"
+                    ):
+                        continue
                     try:
                         self.env.process(equipment.equipment.run_unscheduled(request))
                     except ValueError:
@@ -176,9 +189,7 @@ class RepairManager(FilterStore):
                         pass
                     break
 
-                if equipment.equipment.onsite or equipment.equipment.enroute:
-                    # TODO: have non-tugboat unscheduled maintenance be able to operate like tugboats
-                    # to trigger operatins in the same way
+                if equipment.equipment.dispatched:
                     equipment_mapping.append(equipment_mapping.pop(i))
                     break
 
@@ -265,16 +276,20 @@ class RepairManager(FilterStore):
         # Filter the requests by equipment capability and return the first valid request
         for request in requests:
             if equipment_capability.intersection(request.details.service_equipment):  # type: ignore
+                # If this is the first request for the system, make sure no other servicing
+                # equipment can access it
+                if request.system_id not in self.invalid_systems:
+                    self.invalid_systems.append(request.system_id)
                 return self.get(lambda x: x == requests[0])
 
-        # In case the loop above iterates all the way to the end, nothing was
-        # found so return None. This is probably an error for which an error
-        # should be raised, but this keeps the type hints correct for now.
-
+        # There were no matching equipment requirements to match the equipment
+        # attempting to retrieve its next request
         return None
 
     def get_next_highest_severity_request(
-        self, equipment_capability: Sequence[str], severity_level: Optional[int] = None
+        self,
+        equipment_capability: list[str] | set[str],
+        severity_level: Optional[int] = None,
     ) -> Optional[FilterStoreGet]:
         """Gets the next repair request by ``severity_level``.
 
@@ -293,40 +308,46 @@ class RepairManager(FilterStore):
         if not self.items:
             return None
 
-        equipment_capability = set(equipment_capability)  # type: ignore
+        equipment_capability = set(equipment_capability)
 
-        # Filter the requests by severity level
         requests = self.items
         if severity_level is not None:
+            # Capture only the desired severity level, if specified
             requests = [
-                el
-                for el in self.items
-                if (el.severity_level == severity_level)
-                and (el.system_id not in self.invalid_systems)
+                el for el in self.items if (el.severity_level == severity_level)
             ]
-        if requests == []:
-            return None
+            if requests == []:
+                return None
 
+        # Re-order requests by severity (high to low) and the order they were submitted
+        # Need to ensure that the requests are in submission order in case any get put
+        # back
         requests = sorted(requests, key=lambda x: x.severity_level, reverse=True)
         for request in requests:
-            if equipment_capability.intersection(request.details.service_equipment):  # type: ignore
-                return self.get(lambda x: x == request)
-
-        # This return statement ensures there is always a known return type,
-        # but consider raising an error here if the loop is always assumed to
-        # break when something is found.
+            if request.system_id not in self.invalid_systems:
+                if equipment_capability.intersection(request.details.service_equipment):
+                    self.invalid_systems.append(request.system_id)
+                    return self.get(lambda x: x == request)
 
+        # Ensure None is returned if nothing is found in the loop just as a FilterGet
+        # would if allowed to oeprate without the above looping to identify multiple
+        # criteria and acting on the request before it's processed
         return None
 
-    def halt_requests_for_system(self, system_id: str) -> None:
-        """Disables the ability for servicing equipment to service a specific system.
+    def halt_requests_for_system(self, system: System | Cable) -> None:
+        """Disables the ability for servicing equipment to service a specific system,
+        sets the turbine status to be in servicing, and interrupts all the processes
+        to turn off operations.
 
         Parameters
         ----------
         system_id : str
             The system to disable repairs.
         """
-        self.invalid_systems.append(system_id)
+        if system.id not in self.invalid_systems:
+            self.invalid_systems.append(system.id)
+        system.servicing = self.env.event()
+        system.interrupt_all_subassembly_processes()
 
     def enable_requests_for_system(self, system_id: str) -> None:
         """Reenables service equipment operations on the provided system.
@@ -398,7 +419,7 @@ class RepairManager(FilterStore):
         subassembly_id : str
             Either the ``Subassembly.id`` or the ``Cable.id`` repeated for cables.
         exclude : list[str]
-            A list of ``request_id``s to exclude from the purge. This is a specific use
+            A list of ``request_id`` to exclude from the purge. This is a specific use
             case for the combined cable system/subassembly, but can be to exclude
             certain requests from the purge.
 
@@ -445,8 +466,9 @@ class RepairManager(FilterStore):
         """Creates an updated mapping between the servicing equipment capabilities and
         the number of requests that fall into each capability category (nonzero values only).
         """
-        # mapping = dict(CTV=0, SCN=0, LCN=0, CAB=0, RMT=0, DRN=0, DSV=0)
-        all_requests = [request.details.service_equipment for request in self.items]
-        requests = dict(Counter(chain.from_iterable(all_requests)))
-        # mapping.update(requests)
+        requests = dict(
+            Counter(
+                chain.from_iterable((r.details.service_equipment for r in self.items))
+            )
+        )
         return requests
